import wx

import ColorPanel
import GridSimple
import ListCtrl
import ScrolledWindow

from WidgetStack import WidgetStack


# Some constants
_controllers = ["wx.Choice", "wx.ListBox", "wx.TreeCtrl", "wx.RadioBox",
                "wx.SpinCtrl", "wx.Slider"]

#----------------------------------------------------------------------
def GetMondrianData():
    return \
'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00 \x00\x00\x00 \x08\x06\x00\
\x00\x00szz\xf4\x00\x00\x00\x04sBIT\x08\x08\x08\x08|\x08d\x88\x00\x00\x00qID\
ATX\x85\xed\xd6;\n\x800\x10E\xd1{\xc5\x8d\xb9r\x97\x16\x0b\xad$\x8a\x82:\x16\
o\xda\x84pB2\x1f\x81Fa\x8c\x9c\x08\x04Z{\xcf\xa72\xbcv\xfa\xc5\x08 \x80r\x80\
\xfc\xa2\x0e\x1c\xe4\xba\xfaX\x1d\xd0\xde]S\x07\x02\xd8>\xe1wa-`\x9fQ\xe9\
\x86\x01\x04\x10\x00\\(Dk\x1b-\x04\xdc\x1d\x07\x14\x98;\x0bS\x7f\x7f\xf9\x13\
\x04\x10@\xf9X\xbe\x00\xc9 \x14K\xc1<={\x00\x00\x00\x00IEND\xaeB`\x82' 


def GetMondrianBitmap():
    return wx.BitmapFromImage(GetMondrianImage())


def GetMondrianImage():
    import cStringIO
    stream = cStringIO.StringIO(GetMondrianData())
    return wx.ImageFromStream(stream)


def GetMondrianIcon():
    icon = wx.EmptyIcon()
    icon.CopyFromBitmap(GetMondrianBitmap())
    return icon


#------------------------------------------------------------
# Show how to derive a custom wxLog class
#------------------------------------------------------------

class MyLog(wx.PyLog):

    def __init__(self, textCtrl, logTime=0):

        wx.PyLog.__init__(self)
        self.tc = textCtrl
        self.logTime = logTime


    def DoLogString(self, message, timeStamp):

        if self.tc:
            self.tc.AppendText(message + '\n')


class WidgetStackDemo(wx.Frame):

    def __init__(self, parent, id=wx.ID_ANY,
                 title="WidgetStack wxPython Demo ;-)"):

        wx.Frame.__init__(self, parent, id, title)

        self.splitter = wx.SplitterWindow(self, -1, style=wx.SP_3D|wx.SP_BORDER|
                                          wx.SP_LIVE_UPDATE|wx.SP_3DSASH)

        self.mainPanel = wx.Panel(self.splitter)
        
        self.widgetStack = WidgetStack(self.mainPanel)

        # Add log window
        self.log = wx.TextCtrl(self.mainPanel, wx.ID_ANY, "", size=(-1, 100),
                               style=wx.TE_MULTILINE|wx.TE_READONLY|
                               wx.SUNKEN_BORDER)
        
        wx.Log_SetActiveTarget(MyLog(self.log))

        self.leftPanel = wx.Panel(self.splitter, style=wx.SUNKEN_BORDER)
        
        self.controllerChooser = wx.Choice(self.leftPanel, -1, choices=_controllers)
        self.controllerChooser.SetSelection(0)

        self.BuildWidgetStack()
        self.LayoutItems()

        self.BuildStatusBar()
        self.BuildMenuBar()
        self.SetProperties()
        self.BindEvents()

        self.CreateController(controllerIndex=0, destroyController=False)


    def BuildStatusBar(self):        
        
        statusBar = self.CreateStatusBar(2, wx.ST_SIZEGRIP)
        statusBar.SetStatusWidths([-2, -1])
        # statusbar fields
        statusBar_fields = [("WidgetStack wxPython Demo, Andrea Gavana @ 03 Apr 2007"),
                            ("Welcome To wxPython!")]

        for i in xrange(len(statusBar_fields)):
            statusBar.SetStatusText(statusBar_fields[i], i)
            

    def SetProperties(self):

        self.splitter.SetMinimumPaneSize(150)
        
        xvideo = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_X)
        yvideo = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)

        self.SetSize((3*xvideo/4, 3*yvideo/4))

        self.SetIcon(GetMondrianIcon())  
        self.CenterOnScreen()


    def BindEvents(self):

        self.Bind(wx.EVT_CHOICE, self.OnControllerChoice, self.controllerChooser)
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGING, self.OnPageChanging)
        

    def LayoutItems(self):

        mainSizer = wx.BoxSizer(wx.VERTICAL)
        rightSizer = wx.BoxSizer(wx.VERTICAL)
        leftSizer = wx.BoxSizer(wx.VERTICAL)

        staticLabel = wx.StaticText(self.leftPanel, -1, "Choose Controller")
        staticLabel.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD, False))

        leftSizer.Add(staticLabel, 0, wx.EXPAND|wx.TOP|wx.LEFT|wx.RIGHT, 10)
        leftSizer.Add((0, 4))
        leftSizer.Add(self.controllerChooser, 0, wx.EXPAND|wx.LEFT|wx.RIGHT|wx.BOTTOM, 10)

        self.leftPanel.SetSizer(leftSizer)
        leftSizer.Layout()

        rightSizer.Add(self.widgetStack, 1, wx.EXPAND)
        rightSizer.Add(self.log, 0, wx.EXPAND|wx.TOP, 5)
        self.mainPanel.SetSizer(rightSizer)
        rightSizer.Layout()        

        self.splitter.SplitVertically(self.leftPanel, self.mainPanel, 200)
        
        mainSizer.Add(self.splitter, 1, wx.EXPAND, 0)
        self.SetSizer(mainSizer)
        mainSizer.Layout()
        self.Layout()
        

    def BuildMenuBar(self):

        menuBar = wx.MenuBar()
        fileMenu = wx.Menu()
        helpMenu = wx.Menu()
        
        item = wx.MenuItem(fileMenu, wx.ID_ANY, "E&xit")
        self.Bind(wx.EVT_MENU, self.OnQuit, item)
        fileMenu.AppendItem(item)
                        
        item = wx.MenuItem(helpMenu, wx.ID_ANY, "About")
        self.Bind(wx.EVT_MENU, self.OnAbout, item)
        helpMenu.AppendItem(item)

        menuBar.Append(fileMenu, "&File")
        menuBar.Append(helpMenu, "&Help")

        self.SetMenuBar(menuBar)


    def BuildWidgetStack(self):

        win = self.makeColorPanel(wx.BLUE)
        self.widgetStack.AddWidget(win)
        st = wx.StaticText(win.win, -1,
                          "You can put nearly any type of window here.",
                          (10, 10))

        st.SetForegroundColour(wx.WHITE)
        st.SetBackgroundColour(wx.BLUE)

        win = self.makeColorPanel(wx.RED)
        self.widgetStack.AddWidget(win)

        win = ScrolledWindow.MyCanvas(self.widgetStack)
        self.widgetStack.AddWidget(win)

        win = self.makeColorPanel(wx.GREEN)
        self.widgetStack.AddWidget(win)

        win = GridSimple.SimpleGrid(self.widgetStack, self.log)
        self.widgetStack.AddWidget(win)

        win = ListCtrl.TestListCtrlPanel(self.widgetStack, self.log)
        self.widgetStack.AddWidget(win)

        win = self.makeColorPanel(wx.CYAN)
        self.widgetStack.AddWidget(win)

        win = self.makeColorPanel(wx.NamedColour('Midnight Blue'))
        self.widgetStack.AddWidget(win)

        win = self.makeColorPanel(wx.NamedColour('Indian Red'))
        self.widgetStack.AddWidget(win)


    def makeColorPanel(self, color):

        p = wx.Panel(self.widgetStack, -1)
        win = ColorPanel.ColoredPanel(p, color)
        p.win = win
        
        def OnCPSize(evt, win=win):
            win.SetPosition((0,0))
            win.SetSize(evt.GetSize())

        p.Bind(wx.EVT_SIZE, OnCPSize)

        return p
    

    def OnControllerChoice(self, event):

        self.CreateController(event.GetSelection())
        event.Skip()
        

    def CreateController(self, controllerIndex, destroyController=True):

        self.leftPanel.Freeze()
        leftSizer = self.leftPanel.GetSizer()
        
        if destroyController:
            oldSizerItem = leftSizer.GetChildren()[3]
            leftSizer.Remove(3)
            oldSizerItem.GetWindow().Destroy()

        choices = ["Panel %d"%ii for ii in xrange(1, 10)]
        
        if controllerIndex == 0:   # let's make a choice
            controller = wx.Choice(self.leftPanel, -1, choices=choices)
            controller.Bind(wx.EVT_CHOICE, self.OnChoice)

        elif controllerIndex == 1: # This is a listbox
            controller = wx.ListBox(self.leftPanel, -1, choices=choices)
            controller.Bind(wx.EVT_LISTBOX_DCLICK, self.OnChoice)

        elif controllerIndex == 2: # This is a tree control
            controller = wx.TreeCtrl(self.leftPanel, -1)
            root = controller.AddRoot("Root Item")
            for choice in choices:
                controller.AppendItem(root, choice)
            controller.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnChoice)
            controller.ExpandAll()

        elif controllerIndex == 3: # This is a radio box
            controller = wx.RadioBox(self.leftPanel, -1, "A RadioBox", choices=choices,
                                     majorDimension=1)
            controller.Bind(wx.EVT_RADIOBOX, self.OnChoice)

        elif controllerIndex == 4: # This is a spin control
            controller = wx.SpinCtrl(self.leftPanel, min=1, max=9, initial=1)
            controller.Bind(wx.EVT_SPINCTRL, self.OnChoice)

        elif controllerIndex == 5: # A slider
            controller = wx.Slider(self.leftPanel, -1, value=1, minValue=1, maxValue=9,
                                   style=wx.SL_AUTOTICKS|wx.SL_LABELS)
            controller.Bind(wx.EVT_SCROLL, self.OnChoice)

        leftSizer.Add(controller, 0, wx.EXPAND|wx.ALL, 5)
        leftSizer.Layout()
        self.leftPanel.Thaw()

        
    def OnChoice(self, event):

        object = event.GetEventObject()
        if isinstance(object, wx.TreeCtrl):
            item = event.GetItem()
            if item == object.GetRootItem():
                return
            page = int(object.GetItemText(item).split(" ")[1]) - 1
        elif isinstance(object, wx.RadioBox):
            page = event.GetInt()
        elif isinstance(object, wx.Slider) or isinstance(object, wx.SpinCtrl):
            page = event.GetSelection() - 1
        else:
            page = event.GetSelection()

        self.widgetStack.RaiseWidget(page)


    def OnPageChanging(self, event):

        oldsel = event.GetOldSelection()
        newsel = event.GetSelection()
        self.log.AppendText("Page Changing From: %d To: %d\n"%(oldsel, newsel))
        event.Skip()
        

    def OnQuit(self, event):

    	self.Destroy()


    def OnAbout(self, event):

        msg = "This Is The About Dialog Of The WidgetStack Demo.\n\n" + \
              "Author: Andrea Gavana @ 03 Apr 2007\n\n" + \
              "Please Report Any Bug/Requests Of Improvements\n" + \
              "To Me At The Following Adresses:\n\n" + \
              "andrea.gavana@gmail.com\n" + "gavana@kpo.kz\n\n" + \
              "Welcome To wxPython " + wx.VERSION_STRING + "!!"
              
        dlg = wx.MessageDialog(self, msg, "WidgetStack wxPython Demo",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()
        

def main():

    app = wx.PySimpleApp()
    frame = WidgetStackDemo(None, -1)
    frame.Show()
    app.MainLoop()


if __name__ == "__main__":
    main()
    
        